#!/bin/sh

set -e

TOPDIR=`dirname "$0"`/../..

make -C "${TOPDIR}/tools" streamux

perl -e '
    srand(1);
    $a = (("\x20"x96 . "\x60"x96)x16 . ("\xA0"x96 . "\xE0"x96)x16);
    binmode STDOUT;
    print $a x 5;
    print "\x60\0\xC0" x (2048*5);
    for ($i = 0; $i < 3*2048*10; $i++) {
        print pack("C*",int(rand(256)));
    }' \
    >test.raw

perl -e '
    use Math::Trig;
    use constant TWOPI => 3.1415926535897932 * 2;
    binmode STDOUT;
    for ($i = 0; $i < 29400; $i++) {
        $left = int(10000 * sin(($i/100)*TWOPI));
        $right = int(6000 * sin(($i/50)*TWOPI));
        print pack("vv", $left, $right);
    }' \
    >test.pcm

perl -e '
    use Math::Trig;
    use constant TWOPI => 3.1415926535897932 * 2;
    binmode STDOUT;
    for ($i = 0; $i < 29400; $i++) {
        $left = int(10000 * sin(($i/100)*TWOPI));
        print pack("v", $left);
    }' \
    >test-mono.pcm

# -threads 1 to ensure deterministic behavior.
ffmpeg \
    -threads 1 \
    -codec:v rawvideo -f rawvideo -video_size 64x32 -pixel_format rgb24 \
        -framerate 30 -i test.raw \
    -f s16le -ar 44100 -ac 2 -i test.pcm \
    -codec:v libvpx -codec:a libvorbis -y test.webm
ffmpeg \
    -threads 1 \
    -codec:v rawvideo -f rawvideo -video_size 64x32 -pixel_format rgb24 \
        -framerate 30 -i test.raw \
    -f s16le -ar 44100 -ac 1 -i test-mono.pcm \
    -codec:v libvpx -codec:a libvorbis -y test-mono.webm
ffmpeg \
    -threads 1 \
    -codec:v rawvideo -f rawvideo -video_size 64x32 -pixel_format rgb24 \
        -framerate 30 -i test.raw \
    -codec:v libvpx -y test-nosound.webm

# -codec:a aac seems to insert 1024 samples of silence at the beginning of
# the audio stream, so cut that much from the input data to compensate.
tail -c+4097 test.pcm >test2.pcm
tail -c+2049 test-mono.pcm >test2-mono.pcm
# -strict experimental currently (ffmpeg-1.2.5) needed for -codec:a aac.
ffmpeg \
    -threads 1 \
    -codec:v rawvideo -f rawvideo -video_size 64x32 -pixel_format rgb24 \
        -framerate 30 -i test.raw \
    -f s16le -ar 44100 -ac 2 -i test2.pcm \
    -strict experimental \
    -f mp4 \
        -codec:v libx264 -pix_fmt yuv420p -profile main -level 2.1 -refs 1 \
        -keyint_min 2 -bf 0 -8x8dct 0 -mixed-refs 0 -aud 1 -r 30 -force_fps \
        -codec:a aac -b:a 128k -y test.mp4
ffmpeg \
    -threads 1 \
    -codec:v rawvideo -f rawvideo -video_size 64x32 -pixel_format rgb24 \
        -framerate 30 -i test.raw \
    -f s16le -ar 44100 -ac 1 -i test2-mono.pcm \
    -strict experimental \
    -f mp4 \
        -codec:v libx264 -pix_fmt yuv420p -profile main -level 2.1 -refs 1 \
        -keyint_min 2 -bf 0 -8x8dct 0 -mixed-refs 0 -aud 1 -r 30 -force_fps \
        -codec:a aac -b:a 64k -y test-mono.mp4
ffmpeg \
    -threads 1 \
    -codec:v rawvideo -f rawvideo -video_size 64x32 -pixel_format rgb24 \
        -framerate 30 -i test.raw \
    -f mp4 \
        -codec:v libx264 -pix_fmt yuv420p -profile main -level 2.1 -refs 1 \
        -keyint_min 2 -bf 0 -8x8dct 0 -mixed-refs 0 -aud 1 -r 30 -force_fps \
        -y test-nosound.mp4

ffmpeg \
    -threads 1 \
    -codec:v rawvideo -f rawvideo -video_size 64x32 -pixel_format rgb24 \
        -framerate 30 -i test.raw \
    -f rawvideo \
        -codec:v libx264 -pix_fmt yuv420p -profile main -level 2.1 -refs 1 \
        -keyint_min 2 -bf 0 -8x8dct 0 -mixed-refs 0 -aud 1 -r 30 -force_fps \
        -y test.264
"${TOPDIR}/tools/streamux" test.264 test.pcm 30 >test.str

# Rewrite all the Matroska segment UIDs to 0x000...001 for consistent output.
for f in test.webm test-mono.webm test-nosound.webm; do
    perl -e '
        open F, "+<$ARGV[0]" or die "$ARGV[0]: $!\n";
        binmode F;
        undef $/;
        my $data = <F>;
        my $uid_pos = undef;
        for (my $i = 0; $i < length($data) - 3; $i++) {
            if (substr($data,$i,3) eq "\x73\xA4\x90") {
                die "$ARGV[0]: Multiple SegmentUIDs\n" if defined($uid_pos);
                $uid_pos = $i+3;
            }
        }
        die "$ARGV[0]: SegmentUID not found\n" if !defined($uid_pos);
        seek F, $uid_pos, 0 or die "seek($ARGV[0]): $!\n";
        print F "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\1";
        close F;' \
        "$f"
done

# Make a version of test.webm with a frame rate field of 0 for testing
# code paths that deal with unknown-framerate movies.
perl -pe 's/\x23\xE3\x83\x84\x01\xFC\xA0\x55/\x23\xE3\x83\x84\0\0\0\0/' \
    <test.webm >framerate-0.webm

rm -f test.raw test.pcm test.264 test-mono.pcm test2.pcm test2-mono.pcm
